# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/hamel',
                'doc_host': 'https://hamelsmu.github.io',
                'git_url': 'https://github.com/hamelsmu/hamel',
                'lib_path': 'hamel'},
  'syms': { 'hamel.context': { 'hamel.context._extract_links': ('context.html#_extract_links', 'hamel/context.py'),
                               'hamel.context._fetch_and_save': ('context.html#_fetch_and_save', 'hamel/context.py'),
                               'hamel.context._html_to_markdown_pandoc': ('context.html#_html_to_markdown_pandoc', 'hamel/context.py'),
                               'hamel.context._is_valid_url': ('context.html#_is_valid_url', 'hamel/context.py'),
                               'hamel.context._sanitize_filename': ('context.html#_sanitize_filename', 'hamel/context.py'),
                               'hamel.context.fetch_site_from_sitemap': ('context.html#fetch_site_from_sitemap', 'hamel/context.py')},
            'hamel.oai': { 'hamel.oai._join_url': ('oai.html#_join_url', 'hamel/oai.py'),
                           'hamel.oai._query_git_config': ('oai.html#_query_git_config', 'hamel/oai.py'),
                           'hamel.oai._to_dt': ('oai.html#_to_dt', 'hamel/oai.py'),
                           'hamel.oai.create_openai_plugin_scaffolding': ('oai.html#create_openai_plugin_scaffolding', 'hamel/oai.py'),
                           'hamel.oai.list_models': ('oai.html#list_models', 'hamel/oai.py')}}}
